part.filt <- function(n.part=100,H1=H,Upsilon_draw1=Upsilon_draw,A_draw1=A_draw,Y1=Y,X1=X,pars_var1=pars_var){
  #Calculates the likelihood function of the model
  Lik <- 0
  T <- nrow(Y1)
  M <- ncol(Y1)
  w.0 <- do.call( rbind, rep(list(H1[1,]), n.part) )
  w.part <- matrix(0,n.part,M)
  w.prob <- matrix(0,n.part,1)
  #Y(t)=B X(t) + A e(t)
  #Ainv Y(t)-AinvB H X(t)=e(t)
  Ainv <- solve(t(Upsilon_draw1))
  Yhat <- Y1%*%Ainv
  Xhat <- X1%*%A_draw1%*%Ainv
  uhat <- Yhat-Xhat
  
  for (tt in 1:T){
    #Particle filter
    for (i in 1:n.part){
      w.temp <- pars_var1[1,]+pars_var1[2,]*(w.0[i,]-pars_var1[1,])+rnorm(M,0,sqrt(pars_var1[3,]))
      w.prob[i,] <- sum(dnorm(Yhat[tt,],Xhat[tt,],exp(0.5*w.temp),log=TRUE))#dmvnorm(Y[tt,],(X%*%A_draw)[tt,],Upsilon_draw%*%diag(exp(w.temp))%*%t(Upsilon_draw),log=TRUE)#dnorm(Yhat[tt,],Xhat[tt,],exp(0.5*w.temp),log=TRUE)#uhat[tt,i]
      w.part[i,] <- w.temp
    }
    probs <- exp(w.prob-max(w.prob))/sum(exp(w.prob-max(w.prob)))
    sl.ind <- sample(1:n.part,n.part,probs,replace=TRUE)
    w.0 <- w.part[sl.ind,]
    
    Lik <- Lik+mean(w.prob)
  }
  return(Lik)
}
#print(Lik)

remove_outliers <- function(x, na.rm = TRUE, ...) {
  qnt <- quantile(x, probs=c(.25, .75), na.rm = na.rm, ...)
  H <- 3 * IQR(x, na.rm = na.rm)
  y <- x
  y[x < (qnt[1] - H)] <- NA
  y[x > (qnt[2] + H)] <- NA
  y
}
impulsdtrf <- function(B,smat,nstep)
  ### By:             As emerges from rfvar, neqn x nvar x lags array of rf VAR coefficients.
  ### smat:           nshock x nvar matrix of initial shock vectors.  To produce "orthogonalized
  ###                 impulse responses" it should have the property that crossprod(t(smat))=sigma,
  ###                 where sigma is the Var(u(t)) matrix and u(t) is the rf residual vector.  One
  ###                 way to get such a smat is to set smat=t(chol(sigma)).  To get the smat
  ###                 corresponding to a different ordering, use
  ###                 smat = t(chol(P %*% Sigma %*% t(P)) %*% P), where P is a permutation matrix.
  ###                 To get impulse responses for a structural VAR in the form A(L)y=eps, with
  ###                 Var(eps)=I, use B(L)=-A_0^(-1)A_+(L) (where A_+ is the coefficients on strictly
  ###                 positive powers of L in A), smat=A_0^(-1).
  ###                 In general, though, it is not required that smat be invertible.
### response:       nvar x nshocks x nstep array of impulse responses.
###
### Code written by Christopher Sims, based on 6/03 matlab code.  This version 3/27/04.
### Added dimension labeling, 8/02/04.
{
  ##-----debug--------
  ##browser()
  ##------------------
  neq <- dim(B)[1]
  nvar <- dim(B)[2]
  lags <- dim(B)[3]
  dimnB <- dimnames(B)
  if(dim(smat)[2] != dim(B)[2]) stop("B and smat conflict on # of variables")
  response <- array(0,dim=c(neq,nvar,nstep+lags-1));
  response[ , , lags] <- smat
  response <- aperm(response, c(1,3,2))
  irhs <- 1:(lags*nvar)
  ilhs <- lags * nvar + (1:nvar)
  response <- matrix(response, ncol=neq)
  B <- B[, , seq(from=lags, to=1, by=-1)]  #reverse time index to allow matrix mult instead of loop
  B <- matrix(B,nrow=nvar)
  for (it in 1:(nstep-1)) {
    #browser()
    response[ilhs, ] <- B %*% response[irhs, ]
    irhs <- irhs + nvar
    ilhs <- ilhs + nvar
  }
  ## for (it in 2:nstep)
  ##       {
  ##         for (ilag in 1:min(lags,it-1))
  ##           response[,,it] <- response[,,it]+B[,,ilag] %*% response[,,it-ilag]
  ##       }
  dim(response) <- c(nvar, nstep + lags - 1, nvar)
  response <- aperm(response[ , -(1:(lags-1)), ], c(1, 3, 2)) #drop the zero initial conditions; array in usual format
  dimnames(response) <- list(dimnB[[1]], dimnames(smat)[[2]], NULL)
  ## dimnames(response)[2] <- dimnames(smat)[1]
  ## dimnames(response)[1] <- dimnames(B)[2]
  return(response)
}
atau_post <- function(atau=a_tau,lambda2=lambda2_tau,thetas=theta,k=length(theta),rat=1){
  logpost <- sum(dgamma(thetas,atau,(atau*lambda2/2),log=TRUE))+dexp(atau,rate=rat,log=TRUE)  
  return(logpost)
}
mlag <- function(X,lag)
{
  p <- lag
  X <- as.matrix(X)
  Traw <- nrow(X)
  N <- ncol(X)
  Xlag <- matrix(0,Traw,p*N)
  for (ii in 1:p){
    Xlag[(p+1):Traw,(N*(ii-1)+1):(N*ii)]=X[(p+1-ii):(Traw-ii),(1:N)]
  }
  return(Xlag)  
}
#Creates Dummies
get_dum <- function(lambda,M,p,A_prior,sigma_sq,nabla){
  ydum <- rbind((diag(as.numeric(sigma_sq))%*%A_prior[1:M,])/lambda,matrix(0,(M*p)-M,M),diag(as.numeric(sigma_sq)),matrix(0,1,M))
  J <- diag(1:p)
  xdum <- rbind(cbind(kronecker(J,diag(as.numeric(sigma_sq))/lambda),matrix(0,M*p,1)),cbind(matrix(0,M,M*p),matrix(0,M,1)),c(matrix(0,1,M*p),nabla))
  return(list(X=xdum,Y=ydum))
}
get_ML <- function(lambda,Y_,X_,K,A_prior,sigma_sq,v_prior,T,M,p){
  dum <- get_dum(lambda,M,p,A_prior,sigma_sq,v_prior)  
  X_ <- rbind(X_,dum$X);Y_ <- rbind(Y_,dum$Y)
  
  V_prior <- solve(crossprod(dum$X))
  V_post <- solve(crossprod(X_))
  A_post <- V_post%*%t(X_)%*%Y_
  
  S_post <- crossprod(Y_-X_%*%A_post)  
  
  ml <- -M/2 * (log(det(V_prior))+log(det(crossprod(X_))))+(-(T+M+v_prior-1)/2)*log(det(S_post))
  return(ml)
}
draw_sigma <- function(SSE_Gibbs=crossprod(Y-X%*%A_draw),omega=omega,a_i=0.01,b_i=0.01){
  S <- list()
  M <- ncol(SSE_Gibbs)
  for (kk_2 in 1:M){
    S[[kk_2]]  <-  SSE_Gibbs[1:kk_2,1:kk_2]
  }
  # Set also SSE =(s_[i,j]) & get vectors s_[j]=(s_[1,j] , ... , s_[j-1,j])
  s <- list()
  for (kk_3 in 2:M){
    s[[kk_3 - 1]]  <-  SSE_Gibbs[1:(kk_3 - 1),kk_3]
  }
  
  # Parameters for Eta|omega ~ N_[j-1](0,D_[j]*R_[j]*D_[j]), see eq. (15)
  # Create and update h_[j] matrix
  # If omega_[ij] = 0 => h_[ij] = kappa0, else...
  hh <- vector('list',M-1)
  for (jj in 1:(M-1)){
    hh[[jj]] <- omega[1:jj]
  }
  # D_j = diag(hh_[1j],...,hh_[j-1,j])
  D_j <- vector('list',M-1)
  for (kk_5 in 1:(M-1)) {
    D_j[[kk_5]]  <-  diag(hh[[kk_5]])
  }
  D_j[[1]] <- hh[[kk_5]][1]
  # Now create covariance matrix D_[j]*R_[j]*D_[j], see eq. (15)
  DD_j <- vector('list',M-1)
  for (kk_6 in 1:(M-1)){
    DD_j[[kk_6]] <-  D_j[[kk_6]]#(DD%*%DD);
  }
  # Create B_[i] matrix
  B <- vector('list',M)
  for (rr in 1:M) {
    if (rr == 1){
      B[[rr]]  <-  b_i + 0.5*(SSE_Gibbs[rr,rr]);
    }else{if (rr > 1){
      s_i  <-  s[[rr-1]]
      S_i  <-  S[[rr-1]]
      DiDi  <-  DD_j[[rr-1]]
      B[[rr]]  <-  b_i + 0.5*(SSE_Gibbs[rr,rr] - t(s_i)%*%solve(S_i + solve(DiDi))%*%s_i)
    }
    }
  }
  #Now get B_i from cell array B, and generate (psi_[ii])^2
  B_i  <-  as.matrix(unlist(B));
  psi_ii_sq  <- matrix(0,M,1)
  for (kk_7 in 1:M){
    psi_ii_sq[kk_7,1]  <-  rgamma(1,a_i + 0.5*T,B_i[kk_7,1]) #gamm_rnd(1,1,(a_i + 0.5*T),B_i(1,kk_7));
  }
  # Draw eta|psi,phi,gamma,omega,DATA from the [j-1]-variate
  # NORMAL dist.
  eta  <-  vector('list',M-1)
  for (kk_8 in 1:(M-1)){
    s_i  <-  s[[kk_8]]
    S_i  <-  S[[kk_8]]
    DiDi  <-  DD_j[[kk_8]]
    miu_j  <-  - sqrt(psi_ii_sq[kk_8+1])*(solve(S_i + solve(DiDi))%*%s_i);
    Delta_j  <-  solve(S_i + solve(DiDi))
    
    eta[[kk_8]]  <-  miu_j + t(chol(Delta_j))%*%rnorm(kk_8,0,1);
  }
  
  # Create PSI matrix from individual elements of "psi_ii_sq" and "eta"
  PSI_ALL  <-  matrix(0,M,M)
  for (nn_1 in 1:M){ # first diagonal elements
    PSI_ALL[nn_1,nn_1] <-  sqrt(psi_ii_sq[nn_1,1]);
  }
  for (nn_2 in 1:(M-1)){ # Now non-diagonal elements
    eta_gg  <-  eta[[nn_2]]
    for (nnn in 1:dim(eta_gg)[1]){
      PSI_ALL[nnn,nn_2+1]  <-  eta_gg[nnn]
    }
  }
  # Create SIGMA
  SIGMA  <- solve(PSI_ALL%*%t(PSI_ALL))
  
  return(SIG=SIGMA, PSI=PSI_ALL, eta=eta_gg)
}

get_companion <- function(Beta_,varndxv){
  nn <- varndxv[[1]]
  nd <- varndxv[[2]]
  nl <- varndxv[[3]]
  
  nkk <- nn*nl+nd
  Jm <- matrix(0,nkk,nn)
  MM <- matrix(0,nkk,nkk)
  Jm[1:nn,1:nn] <- diag(nn)
  if (nd==1){
    MM <- rbind(t(Beta_), cbind(diag((nl-1)*nn),matrix(0,(nl-1)*nn,nn+nd)),c(matrix(0,nd,nn*nl),nd))
  }else{
    MM <- rbind(t(Beta_), cbind(diag((nl-1)*nn),matrix(0,(nl-1)*nn,nn+nd)))
  }
  return(list(MM=MM,Jm=Jm))
}

get_V <- function(a_bar_1,a_bar_2,ind1,sigma_sq1,p1,set.cons,M1,K1){
  V_i <- matrix(0,K1,M1)
  
  #this double loop fills the prior covariance matrix
  for (i in 1:M1){ #for each i equation
    for (j in 1:K1){ #for each variable on the rhs
      if (set.cons==1) {
        if (j==K1) {
          V_i[j,i] <- 10^3*sigma_sq1[i,1] #variance on constant,trend, dummies and on ex variables         
        }    
        else if (any(j==ind1[i,])) {
          ll <- which(ind1[i,]==j)
          
          V_i[j,i] <- a_bar_1/(ll^2) #variance on own lags
        }else{
          for (kj in 1:M1){
            if (any(j==ind1[kj,]))
            {
              ll <- kj
            }  
          } 
          V_i[j,i] <- (a_bar_2*sigma_sq1[i,1])/((ll^2)*sigma_sq1[ll,1])
        }
      }       
      else{
        if (any(j==ind1[i,])) {
          ll <- which(ind1[i,]==j)
          
          V_i[j,i] <- a_bar_1/(ll^2) #variance on own lags
        } 
        else
        {
          for (kj in 1:M1){
            if (any(j==ind1[kj,]))
            {
              ll <- kj
            }  
          } 
          V_i[j,i] <- (a_bar_2*sigma_sq1[i,1])/((ll^2)*sigma_sq1[ll,1])
        }
      }
    }
  }  
  return(V_i)
}

